﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 「beatmaniaIIDX専用コントローラプレミアムモデル」用スクリプト (2018/02/26)
// ・ダブルプレイ用に2台接続する場合は、以下の定数「DUAL_CONTROLLER」を1に設定してください。
// ・この時コントローラーの認識順が左と右で逆になることがあるため、
//   その場合は以下の定数「JOYSTICK_1」と「JOYSTICK_2」の値をそれぞれ逆にしてください
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// ■■■■定数■■■■
// (以下をコントローラに合わせて修正してください)


// ダブルプレイに対応するか
// ※2台のコントローラーを接続している場合は1にする
DUAL_CONTROLLER	= 0;

// 2台のコントローラーを使用する場合の、1台目と2台目のDirecInput上でのインデックス(接続順に0～)
JOYSTICK_1		= 0;						// 入れ替わった場合はこれを1、
JOYSTICK_2		= 1;						// これを0にすると良い


// アナログのマイナスとプラス値の範囲
ANALOG_RANGE	= 128;						// ロータリーエンコーダーの分解能

// スクラッチによる選曲時の移動量の倍率(スクラッチを回した時に45度で1項目スクロールする分になるように調整)
SCRATCH_SCALE	= 1 / (100 / 8);			// 100カウントで360度なので45度なら8で割る


// ゲームで使用するボタンの定義
//   [ボタンIDの確認方法]
//    ①「コントロールパネル→デバイスとプリンター→接続しているゲームコントローラーを右クリック→
//   　   ゲームコントローラーの設定→コントローラーを選択→プロパティボタン」でプロパティ画面を表示させる
//    ②コントローラーのボタンを押すとプロパティ画面内のどれかのボタンが光るので、
//      その番号を以下のボタンIDに設定する
//
//   [ボタンの配列(1P/2P共通)]
//       Ｂ  Ｄ  Ｆ
//       □  □  □
//     □  □  □  □
//     Ａ  Ｃ  Ｅ  Ｇ
//
//   ※ボタンにIDを割り当てない場合は0を指定する

// 1P側コントローラの各鍵盤のボタンID
LBTNID_A			= 1;
LBTNID_B			= 2;
LBTNID_C			= 3;
LBTNID_D			= 4;
LBTNID_E			= 5;
LBTNID_F			= 6;
LBTNID_G			= 7;

// 2P側コントローラの各鍵盤のボタンID(Singleモードでは右側でも1P側と同じものが使用される)
RBTNID_A			= 1;
RBTNID_B			= 2;
RBTNID_C			= 3;
RBTNID_D			= 4;
RBTNID_E			= 5;
RBTNID_F			= 6;
RBTNID_G			= 7;

// システムボタン(常に1P側のコントローラーを使用)
//   [上部4ボタンのデフォルト値]
//     □ □ □ □
//     E1 E2 E3 E4
//     │ │ │ └ CANCEL
//     │ │ └─- <no>
//     │ └─── EFFECT
//     └────- START
//
SYSBTNID_START		= 9;				// スタートボタン
SYSBTNID_CANCEL		= 12;				// キャンセルボタン
SYSBTNID_EFFECT		= 10;				// エフェクトボタン
SYSBTNID_UP			= 0;				// カーソル↑
SYSBTNID_DOWN		= 0;				// カーソル↓
SYSBTNID_LEFT		= 0;				// カーソル←
SYSBTNID_RIGHT		= 0;				// カーソル→


// ■■■■ここまで■■■■



// グローバル変数
iLeftOldX			= 0;			// 1P側スクラッチの前回の回転値
iRightOldX			= 0;			// 2P側スクラッチの前回の回転値
iJoyID1				= 0;			// 1台目のジョイスティックのインデックス
iJoyID2				= 1;			// 2台目のジョイスティックのインデックス


// ダブルモードに対応しているか(この中では下記APIは使用不可)
function IsSupportDouble()
	return DUAL_CONTROLLER;
end




/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// API一覧
//	GetJoystickCount()				ジョイスティック数を返す
//  GetJoystick( id )				指定のジョイスティックの入力状態を返す
//										引数
//											id					… ジョイスティックID(0～GetJoystickCount()-1)
//										戻り値
//											x,y,z				… axis position
//											rx,ry,rz,			… axis rotation
//											slider0,slider1		… extra axes positions
//											pov0～pov3			… POV directions
//											btn0～btn31			… 32 buttons
//
//	SetAxisRange( id,min,max )		アナログの稼動範囲の値をセット(全軸が対象となる)
//										引数
//											id					… ジョイスティックID(0～GetJoystickCount()-1)
//											min					… 稼動範囲の最小値(初期値-1000)
//											max					… 稼動範囲の最大値(初期値+1000)
//	SetButton( side,id )			ボタンの押下状態をセット
//										引数
//											side				… サイド(0=1P側、1=2P側)
//											id					… ボタンID(0～7)
//	SetScratch( side,ana )			スクラッチの入力状態をセット
//										引数
//											side				… サイド(0=1P側、1=2P側)
//											ana					… 1フレーム内の移動量
//	SetStartButton()				スタートボタンの押下状態をセット
//	SetCancelButton()				キャンセルボタンの押下状態をセット
//	SetEffectButton()				エフェクトボタンの押下状態をセット
//	SetUpButton()					上ボタンの押下状態をセット
//	SetDownButton()					下ボタンの押下状態をセット
//	SetLeftButton()					左ボタンの押下状態をセット
//	SetRightButton()				右ボタンの押下状態をセット
//
//	※1フレームごとに入力値がリセットされるため、
//	　ボタンの押下状態を維持させるには毎フレーム押下関数を呼び出す必要がある
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// デバイスが開始される前に最初に1度だけ呼び出される
function Initialize()
	trace( "■Initialize" );

	local cnt = GetJoystickCount();

	// ジョイスティックのインデックスを算出
	if( cnt>=2 && DUAL_CONTROLLER ) then
		// ジョイスティックが2台以上でダブルモードに対応する場合は、
		// 左右の各コントローラーのインデックスを定数から参照する
		iJoyID1		= JOYSTICK_1;
		iJoyID2		= JOYSTICK_2;
	end

	// 1P側の入力を処理
	if( cnt>=1 ) then
		// ジョイスティックのアナログの稼動範囲を指定する(未設定時は-1000～+1000)
		SetAxisRange( iJoyID1, -ANALOG_RANGE, ANALOG_RANGE );					// 稼動範囲をクリック数に合わせる
		// スクラッチの現在の値を取得(初期値とする)
		js = GetJoystick( iJoyID1 );											// 1台目のコントローラーの情報を取得
		iLeftOldX = js["x"];													// 現在の位置を保存
	end

	// 2P側の入力を処理
	if( cnt>=2 ) then
		// ジョイスティックのアナログの稼動範囲を指定する(未設定時は-1000～+1000)
		SetAxisRange( iJoyID2, -ANALOG_RANGE, ANALOG_RANGE );					// 稼動範囲をクリック数に合わせる
		// スクラッチの現在の値を取得(初期値とする)
		js = GetJoystick( iJoyID2 );											// 2台目のコントローラーの情報を取得
		iRightOldX = js["x"];													// 現在の位置を保存
	end

end


// 毎フレーム呼び出される入力データの処理
function GetInputData()
	local js;
	local sub;
	local cnt = GetJoystickCount()

	// コントローラーが1台も接続されていなければNG
	if( cnt==0 ) then
		trace( "ジョイスティック無し\n" );
		return;
	end

	// コントローラが1つ以上接続されている場合
	if( cnt>=1 ) then
		// 1P側の入力を処理
		js = GetJoystick( iJoyID1 );			// 1台目のコントローラーの情報を取得

		// ボタン
		if( LBTNID_A>0 && js["btn"..(LBTNID_A-1)]==TRUE ) then
			SetButton( 0,0 );					// Aボタンが押されている
		end
		if( LBTNID_B>0 && js["btn"..(LBTNID_B-1)]==TRUE ) then
			SetButton( 0,1 );					// Bボタンが押されている
		end
		if( LBTNID_C>0 && js["btn"..(LBTNID_C-1)]==TRUE ) then
			SetButton( 0,2 );					// Cボタンが押されている
		end
		if( LBTNID_D>0 && js["btn"..(LBTNID_D-1)]==TRUE ) then
			SetButton( 0,3 );					// Dボタンが押されている
		end
		if( LBTNID_E>0 && js["btn"..(LBTNID_E-1)]==TRUE ) then
			SetButton( 0,4 );					// Eボタンが押されている
		end
		if( LBTNID_F>0 && js["btn"..(LBTNID_F-1)]==TRUE ) then
			SetButton( 0,5 );					// Fボタンが押されている
		end
		if( LBTNID_G>0 && js["btn"..(LBTNID_G-1)]==TRUE ) then
			SetButton( 0,6 );					// Gボタンが押されている
		end


		// 前回からのスクラッチの差分を算出
		if( iLeftOldX<-ANALOG_RANGE/2 && js["x"]>ANALOG_RANGE/2 ) then
			// 以前がマイナス値で今回がプラス値だった場合はマイナス値で回り込み
			sub = js["x"] - (iLeftOldX + ANALOG_RANGE * 2) - 1;
		elseif( iLeftOldX>ANALOG_RANGE/2 && js["x"]<-ANALOG_RANGE/2 ) then
			// 以前がプラス値で今回がマイナス値だった場合はプラス値で回り込み
			sub = js["x"] - (iLeftOldX - ANALOG_RANGE * 2) + 1;
		else
			// それ以外なら回り込んでない
			sub = js["x"] - iLeftOldX;
		end
		iLeftOldX = js["x"];					// 1つ前の値として記録

		// 移動量にスケールを掛けた物を今回の回転量とする
		SetScratch( 0, sub * SCRATCH_SCALE );


		// システムボタン(常に1P側を使用)
		if( SYSBTNID_START>0 && js["btn"..(SYSBTNID_START-1)]==TRUE ) then
			SetStartButton();
		end
		if( SYSBTNID_CANCEL>0 && js["btn"..(SYSBTNID_CANCEL-1)]==TRUE ) then
			SetCancelButton();
		end
		if( SYSBTNID_EFFECT>0 && js["btn"..(SYSBTNID_EFFECT-1)]==TRUE ) then
			SetEffectButton();
		end

		if( SYSBTNID_UP>0 && js["btn"..(SYSBTNID_UP-1)]==TRUE ) then
			SetUpButton();
		end
		if( SYSBTNID_DOWN>0 && js["btn"..(SYSBTNID_DOWN-1)]==TRUE ) then
			SetDownButton();
		end
		if( SYSBTNID_LEFT>0 && js["btn"..(SYSBTNID_LEFT-1)]==TRUE ) then
			SetLeftButton();
		end
		if( SYSBTNID_RIGHT>0 && js["btn"..(SYSBTNID_RIGHT-1)]==TRUE ) then
			SetRightButton();
		end

	end

	// コントローラが2つ以上接続されている場合
	if( cnt>=2 ) then
		// 2P側の入力を処理
		js = GetJoystick( iJoyID2 );			// 2台目のコントローラーの情報を取得

		// ボタン処理
		if( RBTNID_A>0 && js["btn"..(RBTNID_A-1)]==TRUE ) then
			SetButton( 1,0 );					// Aボタンが押されている
		end
		if( RBTNID_B>0 && js["btn"..(RBTNID_B-1)]==TRUE ) then
			SetButton( 1,1 );					// Bボタンが押されている
		end
		if( RBTNID_C>0 && js["btn"..(RBTNID_C-1)]==TRUE ) then
			SetButton( 1,2 );					// Cボタンが押されている
		end
		if( RBTNID_D>0 && js["btn"..(RBTNID_D-1)]==TRUE ) then
			SetButton( 1,3 );					// Dボタンが押されている
		end
		if( RBTNID_E>0 && js["btn"..(RBTNID_E-1)]==TRUE ) then
			SetButton( 1,4 );					// Eボタンが押されている
		end
		if( RBTNID_F>0 && js["btn"..(RBTNID_F-1)]==TRUE ) then
			SetButton( 1,5 );					// Fボタンが押されている
		end
		if( RBTNID_G>0 && js["btn"..(RBTNID_G-1)]==TRUE ) then
			SetButton( 1,6 );					// Gボタンが押されている
		end

		// 前回からのスクラッチの差分を算出
		if( iRightOldX<-ANALOG_RANGE/2 && js["x"]>ANALOG_RANGE/2 ) then
			// 以前がマイナス値で今回がプラス値だった場合はマイナス値で回り込み
			sub = js["x"] - (iRightOldX + ANALOG_RANGE * 2) - 1;
		elseif( iRightOldX>ANALOG_RANGE/2 && js["x"]<-ANALOG_RANGE/2 ) then
			// 以前がプラス値で今回がマイナス値だった場合はプラス値で回り込み
			sub = js["x"] - (iRightOldX - ANALOG_RANGE * 2) + 1;
		else
			// それ以外なら回り込んでない
			sub = js["x"] - iRightOldX;
		end
		iRightOldX = js["x"];					// 1つ前の値として記録

		// 移動量にスケールを掛けた物を今回の回転量とする
		SetScratch( 1, sub * SCRATCH_SCALE );
	end

end
